//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
///////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor             : Xilinx
// \   \   \/    Version            : $Name: i+IP+125372 $
//  \   \        Application        : MIG
//  /   /        Filename           : ads5463_ddrif.v
// /___/   /\    Date Last Modified : $Date: 2007/04/18 13:49:24 $
// \   \  /  \   Date Created       : Mon May 2 2005
//  \___\/\___\
//
// Device      : Virtex-4
// Design Name : DDR2 SERDES
// Purpose     : This module is the top level module for the interface.This
//               instantiates ddrif_main_0,infrastrucutre
//               and idelay_ctrl modules.
///////////////////////////////////////////////////////////////////////////////

//`timescale 1ns/1ps

`ifdef SIMULATE_IN_MODELSIM
`else
	`include "defines.vh"
`endif

module ads5463_ddrif
       (
   input [`data_width-1:0] cntrl0_DDR2_DQ,
   input [`data_width-1:0] cntrl1_DDR2_DQ,
   input  REF_CLK200_IN,
   input  SYS_RESET_IN_N,
   input  cntrl0_DDR2_DQS,

   input		dlyen,
   input  		dlyinc,
   input  		dlyrst,
   input  		dlyclk_sel,
	input	[7:0]	capture_modes,
	input	[7:0]	channel_sel,
   input [`data_width-1:0]	dlydbit_sel,
   input [5:0]	dlyinc_val,

   output CLK,
   output CLK90,
   output CLKDIV_0,
   output CLKDIV_90,
   output RESET0,
   output RESET90,
   output RESET270,   

   output				  dqs_div,
   output [`dq_width-1:0] cntrl0_READ_DATA0,
   output [`dq_width-1:0] cntrl0_READ_DATA1,
   output [`dq_width-1:0] cntrl0_READ_DATA2,
   output [`dq_width-1:0] cntrl0_READ_DATA3
        );
   wire clk_200;
   wire idelay_ctrl_rdy;
   wire rst200;   
   wire [`data_width-1:0] READ_DATA0;
   wire [`data_width-1:0] READ_DATA1;
   wire [`data_width-1:0] READ_DATA2;
   wire [`data_width-1:0] READ_DATA3;
   
   reg			dlyen_sync_r;
   reg  		dlyinc_sync_r;
   reg  		dlyrst_sync_r;
   reg  		dlyclk_sel_sync_r;
   reg			dlyen_p;
   reg  		dlyinc_r;
   reg  		dlyrst_r;
   reg  		dlyclk_sel_r;
   reg			cnt_dq_nz;
   reg			cnt_dqs_nz;
   reg [`data_width-1:0]	dq_idly_inc_r;
   reg [`data_width-1:0]	dlydbit_sel_sync_r;
   reg [5:0]	dlyinc_val_sync_r;
   reg [`data_width-1:0]	dlydbit_sel_r;
   reg [5:0]	dlyinc_val_r;
   reg [5:0]	cnt_dq_r;
   reg [5:0]	cnt_dqs_r;
   reg [`data_strobe_width-1:0]  dqs_idly_inc_r;
   reg [`data_strobe_width-1:0]  dqs_idly_ce_r;
   reg [`data_strobe_width-1:0]  dqs_idly_rst_r;
   reg [`data_strobe_width-1:0]  dqs_idly_ce;
   reg [`data_strobe_width-1:0]  dqs_idly_rst;
   reg [`data_width-1:0]         dq_idly_ce;
   reg [`data_width-1:0]         dq_idly_rst;
   reg [`data_width-1:0]         dq_idly_ce_r;
   reg [`data_width-1:0]         dq_idly_rst_r;
	
	reg [`dq_width-1:0] READ_DATA0_r;
	reg [`dq_width-1:0] READ_DATA1_r;
	reg [`dq_width-1:0] READ_DATA2_r;
	reg [`dq_width-1:0] READ_DATA3_r;
	
assign	cntrl0_READ_DATA0 = READ_DATA0_r;
assign	cntrl0_READ_DATA1 = READ_DATA1_r;
assign	cntrl0_READ_DATA2 = READ_DATA2_r;
assign	cntrl0_READ_DATA3 = READ_DATA3_r;
	
ddrif_top_0     top_00
   (
   .DDR2_DQ		(cntrl0_DDR2_DQ),
   .DDR2_DQ2		(cntrl1_DDR2_DQ),
   .READ_DATA0	(READ_DATA0),
   .READ_DATA1	(READ_DATA1),
   .READ_DATA2	(READ_DATA2),
   .READ_DATA3	(READ_DATA3),
   .dqs_div		(dqs_div),
   .DDR2_DQS	(cntrl0_DDR2_DQS),

   .dqs_idelay_inc	(dqs_idly_inc_r),
   .dqs_idelay_ce	(dqs_idly_ce_r),
   .dqs_idelay_rst	(dqs_idly_rst_r),
   .dq_idelay_inc		(dq_idly_inc_r),
   .dq_idelay_ce	(dq_idly_ce_r),
   .dq_idelay_rst	(dq_idly_rst_r),
   
   //infrastructure signals,
   .CLK             (CLK),
   .CLK90           (CLK90),
   .CLKDIV_0        (CLKDIV_0),
   .CLKDIV_90       (CLKDIV_90),
   .RESET0          (RESET0),
   .RESET90         (RESET90),
   .RESET270        (RESET270)
   );

   ddrif_infrastructure infrastructure0
     (
      .CLK              (CLK),
      .CLK90            (CLK90),
      .CLK200           (clk_200),
      .CLKDIV_90        (CLKDIV_90),
      .CLKDIV_0         (CLKDIV_0),
      .idelay_ctrl_rdy  (idelay_ctrl_rdy),      
      .sys_rst          (RESET0),
      .sys_rst_90       (RESET90),
      .sys_rst_270      (RESET270),      
      .rst200           (rst200),
      .REF_CLK200_IN	(REF_CLK200_IN),
      .SYS_RESET_IN_N	(SYS_RESET_IN_N)
      );

   ddrif_idelay_ctrl idelay_ctrl0
     (
      .CLK200           (clk_200),
      .RESET            (rst200),
      .RDY_STATUS       (idelay_ctrl_rdy)
      );

//-----------------------------------------------------------------------------

reg							dlyen_dly_r;
reg							dlyen_sync_dly_r;
reg							dlyen_sync_dly2_r;
reg							dlyen_syncb_r;
reg							dlyen_syncb_dly_r;
reg	[`dq_width-1:0]	READ_DATA1_D;
reg	[`dq_width-1:0]	READ_DATA2_D;
reg	[`dq_width-1:0]	READ_DATA3_D;


// assign	cntrl0_READ_DATA2 = {2'b0,READ_DATA0[11],1'b0,READ_DATA0[10],1'b0,READ_DATA0[9],1'b0,READ_DATA0[8],1'b0,READ_DATA0[6],1'b0,READ_DATA0[5],1'b0,READ_DATA0[4],1'b0,READ_DATA0[3]};
// assign	cntrl0_READ_DATA3 = {2'b0,READ_DATA1[11],1'b0,READ_DATA1[10],1'b0,READ_DATA1[9],1'b0,READ_DATA1[8],1'b0,READ_DATA1[6],1'b0,READ_DATA1[5],1'b0,READ_DATA1[4],1'b0,READ_DATA1[3]};
//  ADS548x bit wise DDR on data D3.D4,D5,D6,D8,D9,D10,D11  Channels above have zero stuffing so one word can be shifted and or'ed together
//  Channels below should have this data put togehter w/o need for shift and or

always @ (posedge dqs_div)
	begin
	READ_DATA1_D <= READ_DATA1;  // one clock delay for DDR clock edge selection in ini file

	if (capture_modes[0])  // second case (default)is 1ch ADS548x taking 8 pair from middle of the bus
					// first case is 2ch taking lower 7 pair for ch A, upper 7 pair ch B
					// first case inverted because ADS62Pxx inverts LVDS from adc to fpga compared to others
		begin
		if (channel_sel[6] || channel_sel[7]) // special case for ADS59C48 to get two more channels muxed down
			begin
			READ_DATA2_D <= {2'b0,READ_DATA3[2],READ_DATA2[2],READ_DATA3[3],READ_DATA2[3],
						READ_DATA3[4],READ_DATA2[4],READ_DATA3[5],READ_DATA2[5],
						READ_DATA3[6],READ_DATA2[6],READ_DATA3[7],READ_DATA2[7],2'b0};
			READ_DATA3_D <= {2'b0,READ_DATA3[8],READ_DATA2[8],READ_DATA3[9],READ_DATA2[9],
						READ_DATA3[10],READ_DATA2[10],READ_DATA3[11],READ_DATA2[11],
						READ_DATA3[12],READ_DATA2[12],READ_DATA3[13],READ_DATA2[13],2'b0};		
			end
			
		else	// normal case for 2-channel DDR such as P49		
			begin
			READ_DATA2_D <= {2'b0,READ_DATA1[6],READ_DATA0[6],READ_DATA1[5],READ_DATA0[5],
						READ_DATA1[4],READ_DATA0[4],READ_DATA1[3],READ_DATA0[3],
						READ_DATA1[2],READ_DATA0[2],READ_DATA1[1],READ_DATA0[1],
						READ_DATA1[0],READ_DATA0[0]};
			READ_DATA3_D <= {2'b0,READ_DATA1[14],READ_DATA0[14],READ_DATA1[13],READ_DATA0[13],
						READ_DATA1[12],READ_DATA0[12],READ_DATA1[11],READ_DATA0[11],
						READ_DATA1[10],READ_DATA0[10],READ_DATA1[9],READ_DATA0[9],
						READ_DATA1[8],READ_DATA0[8]};
			end

		end
	else
		begin
		if (capture_modes[7])  
			begin  // IDDR has falling edge but 6149 wants rising, so have to shift data
			READ_DATA2_D <= {~READ_DATA0[11],~READ_DATA1_D[11],~READ_DATA0[10],~READ_DATA1_D[10], 
						~READ_DATA0[9],~READ_DATA1_D[9],~READ_DATA0[8],~READ_DATA1_D[8],
						~READ_DATA0[6],~READ_DATA1_D[6],~READ_DATA0[5],~READ_DATA1_D[5],
						~READ_DATA0[4],~READ_DATA1_D[4],~READ_DATA0[3],~READ_DATA1_D[3]};
			end
		else
			begin  //  original default Enzo falling edge to IDDR selection
			READ_DATA2_D <= {~READ_DATA1[11],~READ_DATA0[11],~READ_DATA1[10],~READ_DATA0[10], 
						~READ_DATA1[9],~READ_DATA0[9],~READ_DATA1[8],~READ_DATA0[8],
						~READ_DATA1[6],~READ_DATA0[6],~READ_DATA1[5],~READ_DATA0[5],
						~READ_DATA1[4],~READ_DATA0[4],~READ_DATA1[3],~READ_DATA0[3]};
			end
		READ_DATA3_D <= {16'b0};  // previous revision of TSW1200 app did a shift and or of this ch with ch2.  Make all 0.
		end

	end

always @ (READ_DATA0 or READ_DATA1 or READ_DATA2 or READ_DATA3 or capture_modes or READ_DATA2_D or READ_DATA3_D)
begin
	READ_DATA0_r = ~READ_DATA0;
	READ_DATA1_r = ~READ_DATA1;
	
	if (capture_modes[1])
		begin
		READ_DATA2_r = ~READ_DATA2;
		READ_DATA3_r = ~READ_DATA3;
		end

	else
		begin
		READ_DATA2_r = READ_DATA2_D;
		READ_DATA3_r = READ_DATA3_D;
		end
end


//-----------------------------------------------------------------------------

always @(posedge CLK)
if (RESET0)
		cnt_dq_r <=`DLY 0;
else
	begin
		if(cnt_dq_nz)
			cnt_dq_r <=`DLY cnt_dq_r - 1'b1;
		else if(dlyen)
			cnt_dq_r <=`DLY dlyinc_val;
		else 
			cnt_dq_r <=`DLY 0;
	end

always @(cnt_dq_r)
begin
	cnt_dq_nz = |cnt_dq_r;
end

//-----------------------------------------------------------------------------

genvar m;
generate
	for(m=0;m<`data_width;m=m+1)
	begin	:	gen_dq_idly_ce
		always @(cnt_dq_nz or dlydbit_sel or dlyrst)
		begin
			dq_idly_ce[m] = cnt_dq_nz & dlydbit_sel[m];
			dq_idly_rst[m] = dlyrst & dlydbit_sel[m];
		end
	end
endgenerate

genvar o;
generate
	for(o=0;o<`data_width;o=o+1)
	begin	:	gen_dq_idly_ce_r
		always @(posedge CLK)
		begin
			dq_idly_ce_r[o] <= dq_idly_ce[o];
			dq_idly_rst_r[o] <= dq_idly_rst[o];
			dq_idly_inc_r[o] <= dlyinc;		
		end
	end
endgenerate


always @(cnt_dq_nz or dlyclk_sel)
begin
	dqs_idly_ce	= cnt_dq_nz & dlyclk_sel;
end

always @(dlyrst or dlyclk_sel)
begin
	dqs_idly_rst = dlyrst & dlyclk_sel;
end

always @(posedge CLK)
begin
	dqs_idly_ce_r <= dqs_idly_ce;
	dqs_idly_rst_r <= dqs_idly_rst;
	dqs_idly_inc_r <= dlyinc;
end

//-----------------------------------------------------------------------------

endmodule
