//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
///////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor             : Xilinx
// \   \   \/    Version            : $Name: i+IP+125372 $
//  \   \        Application        : MIG
//  /   /        Filename           : ddrif_data_path_iobs_0.v
// /___/   /\    Date Last Modified : $Date: 2007/04/18 13:49:24 $
// \   \  /  \   Date Created       : Mon May 2 2005
//  \___\/\___\
//
// Device      : Virtex-4
// Design Name : DDR2 SERDES
// Purpose     : This module instantiates ddrif_v4_dq_iob,
//               ddrif_v4_dqs_iob,ddrif_v4_dm_iob,ddrif_idelay_rd_en_io
//               which provides DQ,DQS,DM signals to the memory and enable signals
//               for read data.
///////////////////////////////////////////////////////////////////////////////

//`timescale 1ns/1ps

`ifdef SIMULATE_IN_MODELSIM
`else
	`include "defines.vh"
`endif

module ddrif_data_path_iobs_0
  (
   input                            CLK,
   input                            CLK90,
   input                            CLKDIV_0,
   input                            CLKDIV_90,
   input                            RESET0,
   input                            RESET90,
   input                            RESET270,
   input [`data_strobe_width-1:0]   dqs_idelay_inc,
   input [`data_strobe_width-1:0]   dqs_idelay_ce,
   input [`data_strobe_width-1:0]   dqs_idelay_rst,
   input [`data_width-1:0]          dq_idelay_inc,
   input [`data_width-1:0]          dq_idelay_ce,
   input [`data_width-1:0]          dq_idelay_rst,
   input [`data_width-1:0]          DDR_DQ,
   input [`data_width-1:0]          DDR_DQ2,
   input [`data_strobe_width-1:0]   DDR_DQS, // Add second clock signal for QDR

   output							dqs_div,
   output [`data_width-1:0]         dp_iobs_read_data0,
   output [`data_width-1:0]         dp_iobs_read_data1,
   output [`data_width-1:0]         dp_iobs_read_data2,
   output [`data_width-1:0]         dp_iobs_read_data3
   );



   wire [`data_strobe_width-1:0]  	dqs_bufio_out;
   wire [`data_strobe_width-1:0]    dqs_delayed;
   wire [`data_strobe_width-1:0]    dqs_bufio_out_w;
   wire	[`data_width-1:0]				idly_out;
   wire	[`data_width-1:0]				idly_out2;
   wire                             clk_en;
   wire                             clk_en90;

   assign                           clk_en = ~RESET0;
   assign                           clk_en90 = ~RESET90;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DQS instances
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

   IDELAY #(
		.IOBDELAY_TYPE	("VARIABLE"),
		.IOBDELAY_VALUE	(`DQS_DEFAULT_DLY)
   	)
	idelay_dqs  // For QDR: idelay_dqs[`data_strobe_width-1:0] or just copy twice with [1]
	(
		.O          (dqs_delayed[0]),
		.I          (DDR_DQS[0]),  //ADC Clk (DRY)
		.C          (CLK),
		.CE         (dqs_idelay_ce[0]),
		.INC        (dqs_idelay_inc[0]),
		.RST        (dqs_idelay_rst[0])
	);


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// BUFIO Instances
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

   BUFIO bufio_dqs0
     (
      .O                   (dqs_bufio_out_w[0]),
      .I                   (dqs_delayed[0])
      );

assign #0.595 dqs_bufio_out  = dqs_bufio_out_w;  // Added for simulation only to account for round trip delay and BUFIO/clock route delay

// deserialization factor in DDR mode.
BUFR #(
		.BUFR_DIVIDE("1")
	)
	BUFR_inst(
		.O(dqs_div),
		.CE(1'b1),
		.CLR(1'b0),
		.I(dqs_bufio_out[0])
	);
// synthesis attribute buffer_type of dqs_div is none


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//DQ_IOB instances
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

wire gnd;

assign gnd = 1'b0;

IDELAY	#(
	.IOBDELAY_TYPE	("VARIABLE"),
	.IOBDELAY_VALUE	(`DQ_DEFAULT_DLY)
)
	IDLY_inst_[`data_width-1:0]
	(
		.O		(idly_out[`data_width-1:0]),
		.I		(DDR_DQ[`data_width-1:0]), //ADC Data
		.C		(CLK),
		.CE		(dq_idelay_ce[`data_width-1:0]),
		.INC	(dq_idelay_inc[`data_width-1:0]),
		.RST	(dq_idelay_rst[`data_width-1:0])
	);

	IDDR #(
		.DDR_CLK_EDGE	("SAME_EDGE_PIPELINED"),
		.INIT_Q1		(1'b0),		// Initial value of Q1: 1'b0 or 1'b1
		.INIT_Q2		(1'b0),		// Initial value of Q2: 1'b0 or 1'b1
		.SRTYPE			("SYNC")	// Set/Reset type: "SYNC" or "ASYNC"
		)
	IDDR_inst_[`data_width-1:0]
	(
		.Q1	(dp_iobs_read_data0[`data_width-1:0]),	// 1-bit output for positive edge of clock
		.Q2	(dp_iobs_read_data1[`data_width-1:0]),	// 1-bit output for negative edge of clock
		.C	(~dqs_bufio_out[0]),					// 1-bit clock input //EVM caused this NOT for 5463
		.CE	(1'b1),									// 1-bit clock enable input
		.D	(idly_out[`data_width-1:0]),			// 1-bit DDR data input
		.R	(1'b0),									// 1-bit reset
		.S	(1'b0)									// 1-bit set
		);

IDELAY	#(
	.IOBDELAY_TYPE	("VARIABLE"),
	.IOBDELAY_VALUE	(`DQ_DEFAULT_DLY)
)
	IDLY_inst2_[`data_width-1:0]
	(
		.O		(idly_out2[`data_width-1:0]),
		.I		(DDR_DQ2[`data_width-1:0]), //ADC Data
		.C		(CLK),
		.CE		(dq_idelay_ce[`data_width-1:0]),
		.INC		(dq_idelay_inc[`data_width-1:0]),
		.RST		(dq_idelay_rst[`data_width-1:0])
	);

	IDDR #(
		.DDR_CLK_EDGE	("SAME_EDGE_PIPELINED"),
		.INIT_Q1		(1'b0),		// Initial value of Q1: 1'b0 or 1'b1
		.INIT_Q2		(1'b0),		// Initial value of Q2: 1'b0 or 1'b1
		.SRTYPE			("SYNC")	// Set/Reset type: "SYNC" or "ASYNC"
		)
	IDDR_inst2_[`data_width-1:0]
	(
		.Q1	(dp_iobs_read_data2[`data_width-1:0]),	// 1-bit output for positive edge of clock
		.Q2	(dp_iobs_read_data3[`data_width-1:0]),	// 1-bit output for negative edge of clock
		.C		(~dqs_bufio_out[0]),					// 1-bit clock input //EVM caused this NOT for 5463
		.CE	(1'b1),									// 1-bit clock enable input
		.D		(idly_out2[`data_width-1:0]),			// 1-bit DDR data input
		.R		(1'b0),									// 1-bit reset
		.S		(1'b0)									// 1-bit set
		);


endmodule
